﻿using System;
using System.Threading;
using GHIElectronics.TinyCLR.Devices.Gpio;

namespace TCLR_PANDA3_STEPPER
{
    /// <summary>
    /// A3967 EasyDriver Shield Stepper Motor Driver Module class
    /// </summary>
    /// <remarks>
    /// Thanks to Gus (GHI Electronics) for his code share
    /// </remarks>
    public class EasyStepperDriver
    {
        GpioController gpio = GpioController.GetDefault();
        GpioPin _SleepPin;
        GpioPin _EnablePin;
        GpioPin _DirectionPin;
        GpioPin _StepPin;
        GpioPin _StepModePinOne;
        GpioPin _StepModePinTwo;

        int _StepDelay = 2;
        UInt32 _Steps = 0;
        Mode _StepMode = Mode.Full;
        Direction _StepDirection = Direction.Forward;

        /// <summary>
        /// Get Steps
        /// </summary>
        public UInt32 Steps
        {
            get
            {
                return _Steps;
            }
        }

        /// <summary>
        /// Get direction
        /// </summary>      
        public Direction StepDirection
        {
            get
            {
                return _StepDirection;
            }
            set
            {
                _StepDirection = value;
            }
        }

        /// <summary>
        /// Get Mode
        /// </summary>
        public Mode StepMode
        {
            get
            {
                return _StepMode;
            }
            set
            {
                _StepMode = value;
            }
        }

        /// <summary>
        /// Get time between two step
        /// </summary>
        public int StepDelay
        {
            get
            {
                return _StepDelay;
            }
        }

        /// <summary>
        /// Get if Sleep or not
        /// </summary>
        public bool IsDriverSleep
        {
            get
            {
                if (_SleepPin.Read() == GpioPinValue.High)
                    return true;
                else
                    return false;
            }
        }

        /// <summary>
        /// Get if Enable or Disable
        /// </summary>
        public bool IsOutputsEnable
        {
            get
            {
                if (_EnablePin.Read() == GpioPinValue.Low)
                    return true;
                else
                    return false;
            }
        }

        /// <summary>
        /// Creates an instance of the driver, that only lets you move and choose direction.
        /// </summary>
        /// <param name="directionPin">Control EasyDriver DIR: a digital pin used for direction</param>
        /// <param name="stepPin">Control EasyDriver STEP: a digital pin used for steps</param>
        public EasyStepperDriver(int directionPin, int stepPin)
        {
            _DirectionPin = gpio.OpenPin(directionPin);
            _DirectionPin.SetDriveMode(GpioPinDriveMode.Output);
            _DirectionPin.Write(GpioPinValue.Low);   // Forward
            _StepPin = gpio.OpenPin(stepPin);
            _StepPin.SetDriveMode(GpioPinDriveMode.Output);
        }

        /// <summary>
        /// Creates an instance of the driver, that only lets you move, choose direction and put the controller to sleep
        /// </summary>
        /// <param name="directionPin">Control EasyDriver DIR: a digital pin used for direction</param>
        /// <param name="stepPin">Control EasyDriver STEP: a digital pin used for steps</param>
        /// <param name="sleepPin">Control EasyDriver SLP: a digital pin used for sleep function (disable by default)</param>
        public EasyStepperDriver(int directionPin, int stepPin, int sleepPin) : this(directionPin, stepPin)
        {
            _SleepPin = gpio.OpenPin(sleepPin);
            _SleepPin.SetDriveMode(GpioPinDriveMode.Output);
            _SleepPin.Write(GpioPinValue.Low); // Sleep activé
        }

        /// <summary>
        /// Creates an instance of the driver, that only lets you move, choose direction, sleep, and select step mode
        /// </summary>
        /// <param name="directionPin">Control EasyDriver DIR: a digital pin used for direction</param>
        /// <param name="stepPin">Control EasyDriver STEP: a digital pin used for steps</param>
        /// <param name="sleepPin">Control EasyDriver SLP: a digital pin used for sleep function (disable by default)</param>
        /// <param name="stepModePinOne">Control EasyDriver MS1: a digital pin used for control mode</param>
        /// <param name="stepModePinTwo">Control EasyDriver MS2: a digital pin used for control mode</param>
        public EasyStepperDriver(int directionPin, int stepPin, int sleepPin, int stepModePinOne, int stepModePinTwo) : this(directionPin, stepPin, sleepPin)
        {
            _StepModePinOne = gpio.OpenPin(stepModePinOne); // Huitième de pas
            _StepModePinOne.SetDriveMode(GpioPinDriveMode.Output);
            _StepModePinOne.Write(GpioPinValue.High);
            _StepModePinTwo = gpio.OpenPin(stepModePinTwo);
            _StepModePinTwo.SetDriveMode(GpioPinDriveMode.Output);
            _StepModePinTwo.Write(GpioPinValue.High);
        }

        /// <summary>
        /// Creates an instance of the driver, that only lets you move, choose direction, sleep, select step mode and enable / disable card
        /// </summary>
        /// <param name="DirectionPin">Control EasyDriver DIR PIN: a digital pin used for direction</param>
        /// <param name="StepPin">Control EasyDriver STEP: a digital pin used for steps</param>
        /// <param name="SleepPin">Control EasyDriver SLP: a digital pin used for sleep function (disable par default)</param>
        /// <param name="StepModePinOne">Control EasyDriver MS1: a digital pin used for control mode</param>
        /// <param name="StepModePinTwo">Control EasyDriver MS2: a digital pin used for control mode</param>
        /// <param name="EnablePin">Control EasyDriver ENABLE: a digital pin used for Enable function</param>
        public EasyStepperDriver(int directionPin, int stepPin, int sleepPin, int stepModePinOne, int stepModePinTwo, int enablePin) : this(directionPin, stepPin, sleepPin, stepModePinOne, stepModePinTwo)
        {
            _EnablePin = gpio.OpenPin(enablePin); // Activation des sorties
            _EnablePin.SetDriveMode(GpioPinDriveMode.Output);
            _EnablePin.Write(GpioPinValue.Low);
        }

        /// <summary>
        /// Put the stepper driver to sleep mode
        /// </summary>
        /// <returns>True if the circuit has been asleep</returns>
        public bool Sleep()
        {
            if (_SleepPin != null)
            {
                _SleepPin.Write(GpioPinValue.Low);
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// Wake up the stepper driver
        /// </summary>
        /// <returns>true if circuit have been wake up</returns>
        public bool WakeUp()
        {
            if (_SleepPin != null)
            {
                _SleepPin.Write(GpioPinValue.High);
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// Enable the stepper driver outputs
        /// </summary>
        /// <returns>true if outputs have been enabled</returns>
        public bool EnableOutputs()
        {
            if (_EnablePin != null)
            {
                _EnablePin.Write(GpioPinValue.Low);
                return true;
            }
            else
                return false;
        }

        /// <summary>
        /// Disable the stepper driver outputs
        /// </summary>
        /// <returns>true if outputs have been disabled</returns>
        public bool DisableOutputs()
        {
            if (_EnablePin != null)
            {
                _EnablePin.Write(GpioPinValue.High);
                return true;
            }
            else
                return false;
        }

        /// <summary>
        ///  Moves the stepper motor
        /// </summary>
        /// <param name="steps">Indicate the amount of steps that need to be moved</param>
        /// <param name="direction">Duration between steps</param>
        /// <param name="stepdelay">Delay between steps</param>
        /// <param name="mode">Full, Half, Quarter or OneEighth</param>
        public void Turn(UInt32 steps, Direction direction, int stepdelay = 2, Mode mode = Mode.OneEighth)
        {
            _StepMode = mode; _StepDirection = direction; _StepDelay = stepdelay; _Steps = steps;
            ChangeStepMode(mode);
            ChangeDirection(direction);
            for (UInt32 i = 0; i < _Steps; i++)
            {
                _StepPin.Write(GpioPinValue.High);
                Thread.Sleep(_StepDelay);
                _StepPin.Write(GpioPinValue.Low);
            }
        }

        /// <summary>
        /// Set Direction pin
        /// </summary>
        /// <param name="dir">Forward or Backward</param>
        private void ChangeDirection(Direction dir)
        {
            switch (dir)
            {
                case Direction.Forward:
                    if (_DirectionPin != null)
                        _DirectionPin.Write(GpioPinValue.High);
                    break;
                case Direction.Backward:
                    if (_DirectionPin != null)
                        _DirectionPin.Write(GpioPinValue.Low);
                    break;
            }
        }
        /// <summary>
        /// Set pins MS1 and MS2 
        /// </summary>
        /// <param name="mode">Full, Half, Quarter or OneEighth</param>
        private void ChangeStepMode(Mode mode)
        {
            if (_StepModePinOne != null & _StepModePinTwo != null)
            {
                switch (mode)
                {
                    case Mode.Full:
                        _StepModePinOne.Write(GpioPinValue.Low);
                        _StepModePinTwo.Write(GpioPinValue.Low);
                        break;
                    case Mode.Half:
                        _StepModePinOne.Write(GpioPinValue.High);
                        _StepModePinTwo.Write(GpioPinValue.Low);
                        break;
                    case Mode.Quarter:
                        _StepModePinOne.Write(GpioPinValue.Low);
                        _StepModePinTwo.Write(GpioPinValue.High);
                        break;
                    case Mode.OneEighth:
                        _StepModePinOne.Write(GpioPinValue.High);
                        _StepModePinTwo.Write(GpioPinValue.High);
                        break;
                }
            }
        }

        /// <summary>
        /// Directions are Forward or Backward
        /// </summary>
        public enum Direction
        {
            Forward,
            Backward
        }
        /// <summary>
        /// Modes are Full, Half, Quarter, OneEighth
        /// </summary> 
        public enum Mode
        {
            Full,
            Half,
            Quarter,
            OneEighth
        }
    }
}
